@@ -1,16 +1,47 @@
-#' @rdname tsibble-tidyverse
-#' @usage NULL
-#' @examples
-#' library(tidyr)
-#' # reshaping examples from tidyr
-#' stocks <- tsibble(
-#'   time = as.Date("2009-01-01") + 0:9,
-#'   X = rnorm(10, 0, 1),
-#'   Y = rnorm(10, 0, 2),
-#'   Z = rnorm(10, 0, 4)
-#' )
-#' (stocksm <- stocks %>% gather(stock, price, -time))
-#' stocksm %>% spread(stock, price)
+globalVariables(c("name"))
+
+pivot_longer.tbl_ts <- function(data, cols, names_to = "name", ...) {
+  if (!has_length(names_to)) {
+    abort("`pivot_longer(<tsibble>)` can't accept zero-length `names_to`.")
+  }
+  if (".value" %in% names_to) {
+    abort("`pivot_longer(<tsibble>)` can't accept the special \".value\" in `names_to`.")
+  }
+  new_key <- c(key_vars(data), names_to)
+  vars <- names(eval_select(enquo(cols), data))
+  data <- mutate_index2(data, vars)
+  tbl <- tidyr::pivot_longer(as_tibble(data),
+    cols = !!enquo(cols), names_to = names_to, ...)
+  build_tsibble(tbl,
+    key = !!new_key, index = !!index(data), index2 = !!index2(data),
+    ordered = is_ordered(data), interval = interval(data), validate = FALSE
+  )
+}
+
+pivot_wider.tbl_ts <- function(data, id_cols = NULL, names_from = name, ...) {
+  key_var <- vars_pull(names(data), !!enquo(names_from))
+  if (has_index(key_var, data)) {
+    abort(c(
+      sprintf("Column `%s` (index) can't be widened.", key_var),
+      i = "Please use `as_tibble()` to coerce."
+    ))
+  }
+  key_left <- setdiff(key_vars(data), key_var)
+  new_key <- key_vars(remove_key(data, .vars = key_left))
+
+  tbl <- tidyr::pivot_wider(as_tibble(data),
+    id_cols = !!enquo(id_cols), names_from = !!enquo(names_from), ...)
+  tbl <- retain_tsibble(tbl, new_key, index(data))
+
+  vars <- names(tbl)
+  data <- mutate_index2(data, vars)
+  build_tsibble(
+    tbl,
+    key = !!new_key, index = !!index(data), index2 = !!index2(data),
+    ordered = is_ordered(data), interval = is_regular(data),
+    validate = FALSE
+  )
+}
 
 gather.tbl_ts <- function(data, key = "key", value = "value", ...,
                           na.rm = FALSE, convert = FALSE, factor_key = FALSE) {
@@ -24,7 +55,7 @@ gather.tbl_ts <- function(data, key = "key", value = "value", ...,
       c(quo_name(key), quo_name(value), quo_name(index(data)))
     )
   }
-  vars <- vars_select(names(data), !!!exprs)
+  vars <- names(eval_select(expr(c(...)), data))
   data <- mutate_index2(data, vars)
   tbl <- tidyr::gather(
     as_tibble(data),
@@ -42,9 +73,9 @@ spread.tbl_ts <- function(data, key, value, ...) {
   value <- enexpr(value)
   key_var <- vars_pull(names(data), !!key)
   if (has_index(key_var, data)) {
-    abort(paste_inline(
+    abort(c(
       sprintf("Column `%s` (index) can't be spread.", key_var),
-      "Please use `as_tibble()` to coerce."
+      i = "Please use `as_tibble()` to coerce."
     ))
   }
   key_left <- setdiff(key_vars(data), key_var)
@@ -64,24 +95,25 @@ spread.tbl_ts <- function(data, key, value, ...) {
 }
 
 nest.tbl_ts <- function(.data, ...) {
-  tbl_nest <- tidyr::nest(as_tibble(.data), ...)
-  data_names <- names(.data)
+  data <- .data
+  tbl_nest <- tidyr::nest(as_tibble(data), ...)
+  data_names <- names(data)
   nest_names <- names(tbl_nest)
   nest_vars <- setdiff(data_names, nest_names)
-  if (!has_all_key(nest_vars, .data) && !has_index(nest_vars, .data)) {
+  if (!has_all_key(nest_vars, data) && !has_index(nest_vars, data)) {
     build_tsibble(tbl_nest,
-      key = setdiff(key_vars(.data), nest_vars),
-      index = !!index(.data), validate = FALSE
+      key = setdiff(key_vars(data), nest_vars),
+      index = !!index(data), validate = FALSE
     )
-  } else if (!has_index(nest_vars, .data)) {
-    build_tsibble(tbl_nest, index = !!index(.data), validate = FALSE)
+  } else if (!has_index(nest_vars, data)) {
+    build_tsibble(tbl_nest, index = !!index(data), validate = FALSE)
   } else {
     new_lst <- nest_names[map_lgl(tbl_nest, is_list)]
-    old_lst <- data_names[map_lgl(.data, is_list)]
+    old_lst <- data_names[map_lgl(data, is_list)]
     lst_vars <- setdiff(new_lst, old_lst)
-    .data <- select_tsibble(ungroup(.data), !!!nest_vars, validate = FALSE)
+    data <- remove_key(ungroup(data), nest_vars)
     tbl_nest[[lst_vars]] <- lapply(tbl_nest[[lst_vars]],
-      function(x) update_meta(x, .data))
+      function(x) update_meta(x, data))
     tbl_nest
   }
 }
@@ -107,7 +139,7 @@ unnest_check_tsibble <- function(data, key, index) {
 #' Unnest a data frame consisting of tsibbles to a tsibble
 #'
 #' @description
-#' \lifecycle{questioning}
+#' \lifecycle{deprecated}
 #'
 #' @param data A data frame contains homogenous tsibbles in the list-columns.
 #' @param cols Names of columns to unnest.
@@ -115,6 +147,7 @@ unnest_check_tsibble <- function(data, key, index) {
 #' @keywords internal
 #' @export
 unnest_tsibble <- function(data, cols, key = NULL, validate = TRUE) {
+  # seems not working anymore
   if (!is_installed("tidyr") && utils::packageVersion("tidyr") >= "0.9.0") {
     abort("Package 'tidyr' (>= v1.0.0) required for `unnest_tsibble()`.")
   }
@@ -145,7 +178,7 @@ unnest_tsibble <- function(data, cols, key = NULL, validate = TRUE) {
     idx <- index(tsbl)
   }
 
-  key <- use_id(unnested_data, !!enquo(key))
+  key <- names(eval_select(enquo(key), data = unnested_data))
   idx_chr <- as_string(idx)
   class(unnested_data[[idx_chr]]) <- class(tsbl[[idx_chr]])
   build_tsibble(
@@ -165,3 +198,10 @@ fill.tbl_ts <- function(data, ..., .direction = c("down", "up")) {
 }
 
 fill.grouped_ts <- fill.tbl_ts
+
+drop_na.tbl_ts <- function(data, ...) {
+  res <- NextMethod()
+  update_meta2(res, data, ordered = is_ordered(data), interval = interval(data))
+}
+
+drop_na.grouped_ts <- drop_na.tbl_ts
